{-# LANGUAGE MultiParamTypeClasses #-}
{-# LANGUAGE FlexibleInstances #-}

module Parser1 where

import Data.Char

type Result a = Maybe (a, String)
data Parser a = Parser (String -> Result a)


parse					:: Parser a -> String -> Result a
parse (Parser p) inp 	=  p inp 

-- Basics

failure					:: Parser a
failure					=  Parser $ \inp -> Nothing

item					:: Parser Char
item					=  Parser $ \inp -> case inp of
										[] 		-> Nothing
										(x:xs)	-> Just (x,xs)
							
instance Monad Parser where
  return v  	= Parser $ \inp -> Just (v, inp)
  p >>= f		= Parser $ \inp -> case parse p inp of
								Nothing			-> Nothing
								Just (v, out) 	-> parse (f v) out

-- Sequence example		
psequence 	:: Parser (Char, Char)
psequence 	=  do
				x <- item
				item
				y <- item
				return (x,y)


-- Primitives

sat 	:: (Char -> Bool) -> Parser Char
sat p	=  do
			x <- item
			if p x then return x else failure
				
digit	:: Parser Char
digit	=  sat isDigit	
		
lower	:: Parser Char
lower	=  sat isLower

upper	:: Parser Char
upper	=  sat isUpper

letter	:: Parser Char
letter	=  sat isAlpha

alphanum :: Parser Char
alphanum =  sat isAlphaNum

char 	:: Char -> Parser Char
char x	=  sat(==x)	

string			:: String -> Parser String
string []		=  return []
string (x:xs)	=  do
					char x
					string xs
					return (x:xs)
			
-- Choice
(+++)		:: Parser a -> Parser a -> Parser a
p +++ q		=  Parser $ \inp -> case parse p inp of
									Nothing 	-> parse q inp
									Just x		-> Just x
			
-- Repetition
			
many	:: Parser a -> Parser [a]
many p	=  many1 p +++ return []
		
many1	:: Parser a -> Parser [a]
many1 p	=  do
			v <- p
			vs <- many p
			return (v:vs)
		
-- Identifiers

ident	:: Parser String
ident	=  do
			x <- lower
			xs <- many alphanum
			return (x:xs)
			
nat		:: Parser Int
nat		=  do
			xs <- many1 digit
			return (read xs) -- read :: Read a => String -> a
			
space	:: Parser ()
space	=  do
			many (sat isSpace)
			return ()
		
-- List Parser Example 1

plist1		:: Parser [Int]
plist1		=  do
				string "["
				n <- nat
				ns <- many (
					do
					 string ","
					 nat
					)
				string "]"
				return (n:ns)
		
		
-- Handling spacing

token 		:: Parser a -> Parser a
token p		=  do
				space
				v <- p
				space
				return v
			
identifier	:: Parser String
identifier	=  token ident

natural		:: Parser Int
natural		=  token nat

symbol		:: String -> Parser String
symbol xs	=  token (string xs)


-- List Parser Example 2

plist2		:: Parser [Int]
plist2		=  do
				symbol "["
				n <- natural
				ns <- many (
					do
					 symbol ","
					 natural
					)
				symbol "]"
				return (n:ns)



